@@ -46,6 +46,22 @@ meta_core_get_client_size (Display *xdisplay,
     *height = window->rect.height;
 }
 
+Window
+meta_core_get_client_xwindow (Display *xdisplay,
+                              Window   frame_xwindow)
+{
+  MetaDisplay *display;
+  MetaWindow *window;
+  
+  display = meta_display_for_x_display (xdisplay);
+  window = meta_display_lookup_x_window (display, frame_xwindow);
+
+  if (window == NULL || window->frame == NULL)
+    meta_bug ("No such frame window 0x%lx!\n", frame_xwindow);
+
+  return window->xwindow;
+}
+
 MetaFrameFlags
 meta_core_get_frame_flags (Display *xdisplay,
                            Window   frame_xwindow)
@@ -570,17 +586,17 @@ meta_core_get_menu_accelerator (MetaMenuOp           menu_op,
       name = META_KEYBINDING_MAXIMIZE;
       break;
     case META_MENU_OP_UNSHADE:
-      name = META_KEYBINDING_TOGGLE_SHADE;
-      break;
     case META_MENU_OP_SHADE:
       name = META_KEYBINDING_TOGGLE_SHADE;
       break;
     case META_MENU_OP_UNSTICK:
-      name = META_KEYBINDING_TOGGLE_STICKY;
-      break;
     case META_MENU_OP_STICK:
       name = META_KEYBINDING_TOGGLE_STICKY;
       break;
+    case META_MENU_OP_ABOVE:
+    case META_MENU_OP_UNABOVE:
+      name = META_KEYBINDING_TOGGLE_ABOVE;
+      break;
     case META_MENU_OP_WORKSPACES:
       switch (workspace)
         {
@@ -628,6 +644,18 @@ meta_core_get_menu_accelerator (MetaMenuOp           menu_op,
     case META_MENU_OP_RESIZE:
       name = META_KEYBINDING_BEGIN_RESIZE;
       break;
+    case META_MENU_OP_MOVE_LEFT:
+      name = META_KEYBINDING_MOVE_WORKSPACE_LEFT;
+      break;
+    case META_MENU_OP_MOVE_RIGHT:
+      name = META_KEYBINDING_MOVE_WORKSPACE_RIGHT;
+      break;
+    case META_MENU_OP_MOVE_UP:
+      name = META_KEYBINDING_MOVE_WORKSPACE_UP;
+      break;
+    case META_MENU_OP_MOVE_DOWN:
+      name = META_KEYBINDING_MOVE_WORKSPACE_DOWN;
+      break;
     }
 
   if (name)
@@ -641,7 +669,7 @@ meta_core_get_menu_accelerator (MetaMenuOp           menu_op,
     }
 }
 
-char *
+const char*
 meta_core_get_workspace_name_with_index (Display *xdisplay,
                                          Window   xroot,
                                          int      index)
@@ -654,7 +682,7 @@ meta_core_get_workspace_name_with_index (Display *xdisplay,
   screen = meta_display_screen_for_root (display, xroot);
   g_assert (screen != NULL);
   workspace = meta_screen_get_workspace_by_index (screen, index);
-  return (workspace != NULL) ? workspace->name : NULL;
+  return workspace ? meta_workspace_get_name (workspace) : NULL;
 }
 
 gboolean
@@ -662,6 +690,7 @@ meta_core_begin_grab_op (Display    *xdisplay,
                          Window      frame_xwindow,
                          MetaGrabOp  op,
                          gboolean    pointer_already_grabbed,
+                         int         event_serial,
                          int         button,
                          gulong      modmask,
                          Time        timestamp,
@@ -683,6 +712,7 @@ meta_core_begin_grab_op (Display    *xdisplay,
 
   return meta_display_begin_grab_op (display, screen, window,
                                      op, pointer_already_grabbed,
+                                     event_serial,
                                      button, modmask,
                                      timestamp, root_x, root_y);
 }
