@@ -53,6 +53,7 @@ struct _MetaRectangle
   int height;
 };
 
+typedef struct MetaCompositor  MetaCompositor;
 typedef struct _MetaDisplay    MetaDisplay;
 typedef struct _MetaFrame      MetaFrame;
 typedef struct _MetaKeyBinding MetaKeyBinding;
@@ -152,6 +153,8 @@ struct _MetaDisplay
   Atom atom_net_wm_allowed_actions;
   Atom atom_net_wm_action_move;
   Atom atom_net_wm_action_resize;
+  Atom atom_net_wm_action_fullscreen;
+  Atom atom_net_wm_action_minimize;
   Atom atom_net_wm_action_shade;
   Atom atom_net_wm_action_stick;
   Atom atom_net_wm_action_maximize_horz;
@@ -169,7 +172,9 @@ struct _MetaDisplay
   Atom atom_gnome_panel_action_run_dialog;
   Atom atom_metacity_sentinel;
   Atom atom_net_wm_strut_partial;
-  
+  Atom atom_net_frame_extents;
+  Atom atom_net_request_frame_extents;
+
   /* This is the actual window from focus events,
    * not the one we last set
    */
@@ -179,11 +184,6 @@ struct _MetaDisplay
    */
   MetaWindow *expected_focus_window;
 
-  /* Most recently focused list. Always contains all
-   * live windows.
-   */
-  GList *mru_list;  
-
   guint static_gravity_works : 1;
   
   /*< private-ish >*/
@@ -230,19 +230,24 @@ struct _MetaDisplay
   MetaScreen *grab_screen;
   MetaWindow *grab_window;
   Window      grab_xwindow;
+  gulong      grab_start_serial;
   int         grab_button;
-  int         grab_initial_root_x;
-  int         grab_initial_root_y;
+  int         grab_anchor_root_x;
+  int         grab_anchor_root_y;
+  MetaRectangle grab_anchor_window_pos;
   int         grab_latest_motion_x;
   int         grab_latest_motion_y;
   gulong      grab_mask;
   guint       grab_have_pointer : 1;
   guint       grab_have_keyboard : 1;
+  guint       grab_wireframe_active : 1;
+  guint       grab_was_cancelled : 1;
+  MetaRectangle grab_wireframe_rect;
   MetaRectangle grab_initial_window_pos;
   MetaResizePopup *grab_resize_popup;
   GTimeVal    grab_last_moveresize_time;
   Time        grab_motion_notify_time;
-
+  
   /* we use property updates as sentinels for certain window focus events
    * to avoid some race conditions on EnterNotify events
    */
@@ -291,6 +296,9 @@ struct _MetaDisplay
 
   /* Managed by group-props.c */
   MetaGroupPropHooks *group_prop_hooks;
+
+  /* Managed by compositor.c */
+  MetaCompositor *compositor;
   
 #ifdef HAVE_STARTUP_NOTIFICATION
   SnDisplay *sn_display;
@@ -360,6 +368,7 @@ gboolean meta_display_begin_grab_op (MetaDisplay *display,
                                      MetaWindow  *window,
                                      MetaGrabOp   op,
                                      gboolean     pointer_already_grabbed,
+                                     int          event_serial,
                                      int          button,
                                      gulong       modmask,
                                      Time         timestamp,
@@ -374,9 +383,9 @@ void     meta_display_ungrab_window_buttons  (MetaDisplay *display,
                                               Window       xwindow);
 
 void meta_display_grab_focus_window_button   (MetaDisplay *display,
-                                              Window       xwindow);
+                                              MetaWindow  *window);
 void meta_display_ungrab_focus_window_button (MetaDisplay *display,
-                                              Window       xwindow);
+                                              MetaWindow  *window);
 
 /* make a request to ensure the event serial has changed */
 void     meta_display_increment_event_serial (MetaDisplay *display);
