@@ -56,6 +56,8 @@ typedef enum
   META_SCREEN_RIGHT
 } MetaScreenDirection;
 
+#define META_WIREFRAME_XOR_LINE_WIDTH 5
+
 struct _MetaScreen
 {
   MetaDisplay *display;
@@ -108,10 +110,21 @@ struct _MetaScreen
   guint showing_desktop : 1;
   
   int closing;
+
+  /* gc for XOR on root window */
+  GC root_xor_gc;
+
+  /* Managed by compositor.c; top of stack is first in list */
+  GList *compositor_windows;
+  XID root_picture;
+  XID damage_region;
+  XID trans_pixmap;
+  XID trans_picture;
 };
 
 MetaScreen*   meta_screen_new                 (MetaDisplay                *display,
-                                               int                         number);
+                                               int                         number,
+                                               Time                        timestamp);
 void          meta_screen_free                (MetaScreen                 *screen);
 void          meta_screen_manage_all_windows  (MetaScreen                 *screen);
 MetaScreen*   meta_screen_for_x_screen        (Screen                     *xscreen);
