@@ -193,7 +193,9 @@ window_is_fullscreen_size (MetaWindow *window)
       
       meta_window_get_work_area_current_xinerama (window, &workarea);
       if (window->rect.x <= workarea.x &&
-          window->rect.y <= workarea.y)
+          window->rect.y <= workarea.y &&
+	  window->rect.x + window->rect.width >= workarea.x + workarea.width &&
+	  window->rect.y + window->rect.height >= workarea.y + workarea.height) 
         return TRUE;
     }
   
@@ -207,7 +209,9 @@ window_is_fullscreen_size (MetaWindow *window)
           
           meta_window_get_work_area_current_xinerama (window, &workarea);
           if (window->rect.x <= workarea.x &&
-              window->rect.y <= workarea.y) 
+              window->rect.y <= workarea.y &&
+	      window->rect.x + window->rect.width >= workarea.x + workarea.width &&
+	      window->rect.y + window->rect.height >= workarea.y + workarea.height) 
             return TRUE;
         }
       
@@ -260,9 +264,11 @@ get_standalone_layer (MetaWindow *window)
                                      is_focused_foreach,
                                      &focused_transient);
 
-      if ((window->has_focus || focused_transient ||
-           (window == window->display->expected_focus_window)) &&
-          (window->fullscreen || window_is_fullscreen_size (window)))
+      if (window->wm_state_below)
+        layer = META_LAYER_BOTTOM;
+      else if ((window->has_focus || focused_transient ||
+                (window == window->display->expected_focus_window)) &&
+               (window->fullscreen || window_is_fullscreen_size (window)))
         layer = META_LAYER_FULLSCREEN;
       else if (window->wm_state_above)
         layer = META_LAYER_DOCK;
